/*
 * gsampler-database-instinfo.h
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 *
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gsampler-database-instinfo.h"
#include "gsampler-window.h"
#include "gsampler-utils.h"

#define UI_FILE "gsampler-database-instinfo.ui"

G_DEFINE_TYPE (GsamplerDatabaseInstinfo, gsampler_database_instinfo, GTK_TYPE_DIALOG);

enum {
	GSAMPLER_DATABASE_INST_DUMMY_PROPERTY,
	GSAMPLER_DATABASE_INST_INFO,
	GSAMPLER_DATABASE_INST_LAST
};

static GParamSpec *dev_pops[GSAMPLER_DATABASE_INST_LAST];

static void
gsampler_database_instinfo_set_label (GsamplerDatabaseInstinfo *self, GtkWidget *label, const gchar* text)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (label != NULL);

	if (text != NULL)
		gtk_label_set_text ((GtkLabel*) label, text);
}

static void
gsampler_database_instinfo_desc_change (GsamplerDatabaseInstinfo *self, gpointer data)
{
	gchar *new_desc = NULL;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	GsamplerClient *client = NULL;
	g_return_if_fail (self != NULL);

	gtk_text_buffer_get_start_iter (self->priv->desc_buffer, &start);
	gtk_text_buffer_get_end_iter (self->priv->desc_buffer, &end);
	new_desc = gtk_text_buffer_get_text (self->priv->desc_buffer, &start, &end, TRUE);
	client = gsampler_client_get_instance ();
	if (!gsampler_client_db_set_instrument_desc (client,
	                                             g_strconcat (self->priv->info->db_path, self->priv->info->name, NULL), 
	                                             new_desc))
	{
		g_warning ("file %s: line %d: error: can't update description", __FILE__, __LINE__);
	}
	g_object_unref (client);
	g_free (new_desc);
}

static void 
gsampler_database_instinfo_get_property (GObject *object, guint property_id, 
                                         GValue *value, GParamSpec *pspec) {
	GsamplerDatabaseInstinfo * self;
	self = GSAMPLER_DATABASE_INSTINFO (object);
	g_return_if_fail (self != NULL);
	
	switch (property_id) {
		case GSAMPLER_DATABASE_INST_INFO:
			g_value_set_boxed (value, gsampler_db_inst_info_dup (self->priv->info));
		break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void 
gsampler_database_instinfo_set_property (GObject *object, guint property_id,
                                         const GValue *value, GParamSpec *pspec) {
	GsamplerDatabaseInstinfo *self;
	GsamplerDbInstInfo *inst_info = NULL;
	self = GSAMPLER_DATABASE_INSTINFO (object);
	g_return_if_fail (self != NULL);
			  
	switch (property_id) {
		case GSAMPLER_DATABASE_INST_INFO:
			inst_info = g_value_get_boxed (value);
			if (inst_info != NULL)
				self->priv->info = gsampler_db_inst_info_dup (inst_info);
		break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static GObject *
gsampler_database_instinfo_constructor (GType type, guint n_construct_properties,
                                        GObjectConstructParam * construct_properties) 
{
	GObject *obj = NULL;
	GsamplerDatabaseInstinfo *self = NULL;
	GtkWidget *dialog_box = NULL;
	GtkWidget *info_box = NULL;
	GtkWidget *file_lbl = NULL;
	GtkWidget *name_lbl = NULL;
	GtkWidget *dbpath_lbl = NULL;
	GtkWidget *index_lbl = NULL;
	GtkWidget *size_lbl = NULL;
	GtkWidget *format_lbl = NULL;
	GtkWidget *type_lbl = NULL;
	GtkWidget *desc_but = NULL;
	GtkWidget *created_lbl = NULL;
	GtkWidget *modified_lbl = NULL;
	GtkWidget *product_lbl = NULL;
	GtkWidget *artists_lbl = NULL;
	GtkWidget *keywords_lbl = NULL;

	gchar *load_file = NULL;
	GtkBuilder *builder = NULL;
	GError* error = NULL;
	
	obj = G_OBJECT_CLASS (gsampler_database_instinfo_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_DATABASE_INSTINFO (obj);

	gtk_window_set_title ((GtkWindow*) self, "Instrument Properties");
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
	dialog_box = gtk_dialog_get_content_area ((GtkDialog*) self);

	load_file = gsampler_utils_get_data_file (UI_FILE);
	if (load_file != NULL)
	{
		builder = gtk_builder_new ();
		gtk_builder_add_from_file (builder, load_file, &error);
		if (error != NULL)
		{
			g_warning ("Couldn't load builder file: %s", error->message);
			g_error_free (error);
		}
		info_box = GTK_WIDGET (gtk_builder_get_object (builder, "box1"));
		file_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "file_lbl"));
		name_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "name_lbl"));
		dbpath_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "dbpath_lbl"));
		index_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "index_lbl"));
		size_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "size_lbl"));
		format_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "format_lbl"));
		type_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "type_lbl"));
		self->priv->desc_buffer = GTK_TEXT_BUFFER (gtk_builder_get_object (builder, "desc_buffer"));
		desc_but = GTK_WIDGET (gtk_builder_get_object (builder, "desc_but"));
		created_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "created_lbl"));
		modified_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "modified_lbl"));
		product_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "product_lbl"));
		artists_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "artists_lbl"));
		keywords_lbl = GTK_WIDGET (gtk_builder_get_object (builder, "keywords_lbl"));
		
		gtk_box_pack_start ((GtkBox*) dialog_box, info_box, TRUE, TRUE, 5);

		//TODO populate
		gsampler_database_instinfo_set_label (self, file_lbl, self->priv->info->file);
		gsampler_database_instinfo_set_label (self, name_lbl, self->priv->info->name);
		gsampler_database_instinfo_set_label (self, dbpath_lbl, self->priv->info->db_path);
		gsampler_database_instinfo_set_label (self, index_lbl, g_strdup_printf ("%d", self->priv->info->number));
		gsampler_database_instinfo_set_label (self, size_lbl, g_strdup_printf ("%0.1f MB", self->priv->info->size));
		gsampler_database_instinfo_set_label (self, format_lbl, g_strdup_printf ("%s version %s", self->priv->info->format_family,self->priv->info->format_version));
		gsampler_database_instinfo_set_label (self, type_lbl, self->priv->info->type);
		gtk_text_buffer_set_text (self->priv->desc_buffer, self->priv->info->desc, -1);
		gsampler_database_instinfo_set_label (self, created_lbl, self->priv->info->created);
		gsampler_database_instinfo_set_label (self, modified_lbl, self->priv->info->modified);
		gsampler_database_instinfo_set_label (self, product_lbl, self->priv->info->product);
		gsampler_database_instinfo_set_label (self, artists_lbl, self->priv->info->artists);
		gsampler_database_instinfo_set_label (self, keywords_lbl, self->priv->info->keywords);

		g_signal_connect_swapped (desc_but, "clicked", (GCallback) gsampler_database_instinfo_desc_change, self);
		
		g_object_unref (builder);
		g_free (load_file);
	}
	
	return obj;
}

static void
gsampler_database_instinfo_init (GsamplerDatabaseInstinfo *object)
{
	object->priv = GSAMPLER_DATABASE_INSTINFO_GET_PRIVATE (object);
}

static void
gsampler_database_instinfo_finalize (GObject *object)
{
	GsamplerDatabaseInstinfo * self;
	self = GSAMPLER_DATABASE_INSTINFO (object);
	gsampler_db_inst_info_free (self->priv->info);

	G_OBJECT_CLASS (gsampler_database_instinfo_parent_class)->finalize (object);
}

static void
gsampler_database_instinfo_class_init (GsamplerDatabaseInstinfoClass *klass)
{
	gsampler_database_instinfo_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerDatabaseInstinfoPrivate));
	G_OBJECT_CLASS (klass)->get_property = gsampler_database_instinfo_get_property;
	G_OBJECT_CLASS (klass)->set_property = gsampler_database_instinfo_set_property;
	G_OBJECT_CLASS (klass)->constructor = gsampler_database_instinfo_constructor;
	G_OBJECT_CLASS (klass)->finalize = gsampler_database_instinfo_finalize;

	dev_pops[GSAMPLER_DATABASE_INST_INFO] = g_param_spec_boxed ("info", "Info", "Gsampler database instrument info",
	                                                   GSAMPLER_TYPE_DB_INST_INFO,
	                                                   G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
	g_object_class_install_property (G_OBJECT_CLASS (klass),
	                                 GSAMPLER_DATABASE_INST_INFO, 
	                                 dev_pops[GSAMPLER_DATABASE_INST_INFO]);
}

GsamplerDatabaseInstinfo* 
gsampler_database_instinfo_new (GsamplerDbInstInfo *inst_info, GtkWindow *parent) 
{
	GsamplerDatabaseInstinfo* dialog = NULL;
	g_return_val_if_fail (inst_info != NULL, NULL);
	
	dialog = g_object_new (GSAMPLER_TYPE_DATABASE_INSTINFO, "info", inst_info, NULL);
	if (parent != NULL)
	{
		gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
	
		gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
	}
	return dialog;
}